from pathlib import Path

from PIL import Image


def get_root():
    return Path(__file__).parent.absolute()


def prune_by_resolution(input_file, resolution):
    size = (0, 0)
    with Image.open(input_file) as im:
        size = im.size

    res = tuple(map(int, resolution.split("x")))

    if size < res:
        input_file.unlink()
        print(f"Removed {input_file}, resolution: {size} < {res}")



from pinterest_dl import cli_parser, downloader, io, scraper, utils


def run_download(img_urls, output, verbose):
    fallback_urls = [i.replace("/originals/", "/736x/") for i in img_urls]
    return downloader.download_concurrent_with_fallback(
        img_urls, output, fallback_urls, verbose=verbose
    )


def run_prune(local_images, min_resolution):
    if min_resolution:
        for i in local_images:
            utils.prune_by_resolution(i, min_resolution)


def run_scrape(
    url,
    threshold,
    firefox,
    output,
    write,
    persistence,
    incognito,
    dry_run,
    verbose,
    min_resolution,
):
    browser = scraper.Browser().Chrome(
        exe_path=utils.get_root().joinpath("bin", "chromedriver.exe"),
        incognito=incognito,
    )
    if firefox:
        browser = scraper.Browser().Firefox()
    try:
        pin_scraper = scraper.Pinterest(browser)
        img_urls = pin_scraper.scrape(
            url,
            threshold=threshold,
            presistence=persistence,
            verbose=verbose,
        )
    finally:
        browser.close()

    print(f"Found {len(img_urls)} urls")
    if write:
        io.write_json(img_urls, write, indent=4)
    if not dry_run:
        downloaded_files = run_download(img_urls, output, verbose)

        # post download
        run_prune(downloaded_files, min_resolution)
    else:
        for i in img_urls:
            print(i)


def main():
    parser = cli_parser.get_parser()
    args = parser.parse_args()

    if args.cmd == "scrape":
        run_scrape(
            args.url,
            args.threshold,
            args.firefox,
            args.output,
            args.write,
            args.persistence,
            args.incognito,
            args.dry_run,
            args.verbose,
            args.resolution,
        )
        print("\nDone.")
    elif args.cmd == "download":
        img_list = io.read_json(args.url_list)
        downloaded_files = run_download(img_list, args.output, args.verbose)
        run_prune(downloaded_files, args.resolution)
        print("\nDone.")
    else:
        parser.print_help()


if __name__ == "__main__":
    main()



import concurrent.futures
from pathlib import Path

import requests


def download(url: str, output_dir, chunk_size=2048, verbose=False):
    if isinstance(url, str):
        req = requests.get(url)
        req.raise_for_status()

        filename = Path(url).name
        outfile = Path.joinpath(Path(output_dir), filename)
        # create directory if not exist
        outfile.parent.mkdir(parents=True, exist_ok=True)
        with open(outfile, "wb") as payload:
            for chunk in req.iter_content(chunk_size):
                payload.write(chunk)
        print(f"Downloaded {filename}")
        return outfile
    else:
        print("URL must be a string.")


def download_with_fallback(url: str, output_dir, fallback_url, chunk_size=2048, verbose=False):
    try:
        return download(url, output_dir, chunk_size, verbose)
    except requests.exceptions.HTTPError:
        return download(fallback_url, output_dir, chunk_size, verbose)


def download_concurrent(urls: list, output_dir, chunk_size=2048, verbose=False):
    results = []
    with concurrent.futures.ThreadPoolExecutor() as executor:
        futures = [executor.submit(download, url, output_dir, chunk_size, verbose) for url in urls]
        for future in concurrent.futures.as_completed(futures):
            results.append(future.result())
    return results


def download_concurrent_with_fallback(
    urls: list, output_dir, fallback_urls, chunk_size=2048, verbose=False
):
    results = []
    with concurrent.futures.ThreadPoolExecutor() as executor:
        futures = [
            executor.submit(
                download_with_fallback, url, output_dir, fallback_url, chunk_size, verbose
            )
            for url, fallback_url in zip(urls, fallback_urls)
        ]
        for future in concurrent.futures.as_completed(futures):
            results.append(future.result())
    return results



import copy
import os
import random
import socket
import time
from pathlib import Path

from selenium import webdriver
from selenium.common.exceptions import StaleElementReferenceException
from selenium.webdriver.common.by import By
from selenium.webdriver.common.keys import Keys
from selenium.webdriver.remote.webdriver import WebDriver
from selenium.webdriver.remote.webelement import WebElement

from pinterest_dl import downloader, driver_installer, utils


def randdelay(a, b):
    time.sleep(random.uniform(a, b))


class Browser(object):
    def __init__(self):
        self.browser = None

    def Chrome(self, image_enable=False, incognito=False, exe_path="chromedriver.exe"):
        if not os.path.exists(exe_path):
            driver_installer.install_chrome_driver(
                Path(exe_path).parent, version="123.0.6312.86", platform="win64"
            )

        service = webdriver.chrome.service.Service(exe_path)
        chrome_options = webdriver.ChromeOptions()
        # Disable images
        # https://scrapeops.io/selenium-web-scraping-playbook/python-selenium-disable-image-loading/
        chrome_options.add_argument(
            "--blink-settings=imagesEnabled=true"
            if image_enable
            else "--blink-settings=imagesEnabled=false"
        )
        if incognito:
            chrome_options.add_argument("--incognito")
        browser = webdriver.Chrome(options=chrome_options, service=service)
        return browser

    def Firefox(self, image_enable=False, incognito=False):
        firefox_profile = webdriver.FirefoxProfile()
        # Disable images
        if image_enable:
            firefox_profile.set_preference("permissions.default.image", 1)
        else:
            firefox_profile.set_preference("permissions.default.image", 2)
        if incognito:
            firefox_profile.set_preference("browser.privatebrowsing.autostart", True)
        browser = webdriver.Firefox(firefox_profile=firefox_profile)
        return browser

    def _download_chrome_driver_not_exist(self, exe_path):
        if not utils.file_exists(exe_path):
            print("Chrome driver does not exist. Downloading...")

            downloader.download_curl()
        else:
            print("Chrome driver exists.")


class Pinterest(object):
    def __init__(self, browser=None):
        self.browser: WebDriver = browser

    # currently not used
    def login(self, email, password):
        self.browser.get("https://www.pinterest.com.au/login/")
        email_field = self.browser.find_element(By.ID, "email")
        email_field.send_keys(email)
        password_field = self.browser.find_element(By.ID, "password")
        password_field.send_keys(password)
        randdelay(1, 2)  # delay between 1 and 2 seconds
        password_field.send_keys(Keys.RETURN)

    def scrape(
        self,
        url,
        threshold=20,
        presistence=120,
        verbose=False,
    ):
        final_results = []
        previous_divs = []
        tries = 0

        try:
            self.browser.get(url)
            while threshold > 0:
                try:
                    divs = self.browser.find_elements(By.CSS_SELECTOR, "div[data-test-id='pin']")
                    if divs == previous_divs:
                        tries += 1
                    else:
                        tries = 0
                    if tries > presistence:
                        if verbose:
                            print("Exiting: persistence exceeded")
                        return final_results

                    for div in divs:
                        # if div is an ad, skip
                        if self._is_div_ad(div):
                            continue
                        images = div.find_elements(By.TAG_NAME, "img")
                        for image in images:
                            src = image.get_attribute("src")
                            if src and "/236x/" in src:
                                src = src.replace("/236x/", "/originals/")
                                final_results.append(src)
                                if verbose:
                                    print(src)
                    previous_divs = copy.copy(divs)  # copy to avoid reference
                    final_results = list(set(final_results))  # remove duplicates

                    # Scroll down
                    dummy = self.browser.find_element(By.TAG_NAME, "a")
                    dummy.send_keys(Keys.PAGE_DOWN)
                    randdelay(1, 2)  # delay between 1 and 2 seconds
                    threshold -= 1

                except StaleElementReferenceException:
                    print("StaleElementReferenceException")
                    threshold -= 1
        except (socket.error, socket.timeout):
            print("Socket Error")
        except KeyboardInterrupt:
            return final_results
        return final_results

    def _is_div_ad(self, div: WebElement):
        """Check if div is an ad.

        Args:
            div (webElement): div element.
        """
        ads_svg_path = "M12 9a3 3 0 1 0 0 6 3 3 0 0 0 0-6M3 9a3 3 0 1 0 0 6 3 3 0 0 0 0-6m18 0a3 3 0 1 0 0 6 3 3 0 0 0 0-6"
        svg_elements = div.find_elements(By.TAG_NAME, "svg")
        for svg in svg_elements:
            if ads_svg_path in svg.get_attribute("innerHTML"):
                return True
